<?php 
/**
 * ====================================================================================
 *                           PREMIUM URL SHORTENER (c) KBRmedia
 * ----------------------------------------------------------------------------------
 * @copyright This software is exclusively sold at CodeCanyon.net. If you have downloaded this
 *  from another site or received it from someone else than me, then you are engaged
 *  in an illegal activity. You must delete this software immediately or buy a proper
 *  license from http://gempixel.com/buy/short.
 *
 *  Thank you for your cooperation and don't hesitate to contact me if anything
 * ====================================================================================
 *
 * @author KBRmedia (http://gempixel.com)
 * @link http://gempixel.com 
 * @package Premium URL Shortener
 * @subpackage Shorten Request Handler (Short.class.php)
 */
class Short extends App{
	/**
	 * The length of alias for each short URL
	 * @var integer
	 * @since v2.0
	 */
	protected $alias_length=5;
	/**
	 * The return URL Format. Default = Main or Second. URL. To use subdomains, use http://{?}.YOURDOMAIN.com
	 * @var string
	 * @since v2.0	 
	 */	
	protected $return_url=NULL;
	/**
	 * Webpage screenshot generator
	 * @var string, Possible value: "pagepeeker" or your custom API using @URL@ as the placeholder
	 * @see image() function
	 * @since v2.0	 
	 */	
	protected $thumb_provider="pagepeeker";
	/**
	 * Redirect Override
	 * @var boolean, If TRUE, user can use $_GET["r"] to override. E.g. ?r=splash or ?r=frame
	 * @since v3.0
	 */	
	private $redirect_override=FALSE;
	/**
	 * Self-Shrotening
	 * @var boolean, If TRUE, it will be possible to shorten the URL of the domain where this script is located.
	 * @since v3.0
	 */	
	private $self_shortening=FALSE;

	public function __construct($db,$config){
		$this->db=$db;
		$this->config=$config;   		
    $this->check();		
	}
  public function __destruct(){
		unset($this->db,$this->config);
	}
	/**
	 * Main Shorten URL
	 * @since v3.0
	 */
	public function add($array=array(), $api=FALSE){
		// Kill the bots
		if(Main::bot()) die();
		
		if(empty($array)){
			$array=$_POST;
		}

		$url=$this->validate(Main::clean($array["url"],3,FALSE));

		// Get domain name
		$domain=$this->domain($url);
		
		// Check if private
		if($this->config["private"] && !$this->logged_id) return array('error' => 1, 'msg' => e('This service is meant to be used internally only.'));

		// Validate Request
		if(empty($url) || !$url) return array('error' => 1, 'msg' => e('Please enter a valid URL.'));

		// Prevent self-shortening
		if($domain==$this->domain($this->config["url"]) && !$this->self_shortening) return  array('error' => 1, 'msg' => 'You cannot shorten URLs of this website.');		

		// Check domain is blacklisted
		if($this->blacklist_domain($url)) return  array('error' => 1, 'msg' => e('This domain name has been blacklisted.'));	

		// Checks URL with Google
		if(!$this->safe($url)) return  array('error' => 1, 'msg' => e('URL is suspected to contain malwares according to Google.'));

		// Match the domain to the list of keywords
		if($this->config["adult"] && $this->blacklist_keywords($url)) return  array('error' => 1, 'msg' => e('Adult and warez websites are not allowed.'));	

		// Validate selected domain name
		if(isset($array["domain"]) && $this->validate_domain_names(trim($array["domain"]))){
			$this->config["url"]=trim($array["domain"]);
		}

		// Check Captcha
		if($this->config["captcha"] && !$this->logged_id && !isset($_GET["bookmark"])){
			$captcha=Main::check_captcha($array);
			if($captcha!='ok'){
				return array('error' => 1, 'msg' => $captcha,'html'=>'captcha');
			}
		}	
		// Generate formatted list of countries
		if(!empty($array['country'][0]) && !empty($array['target'][0])){
			foreach ($array['country'] as $i => $country) {
				if(!empty($country) && !empty($array['target'][$i])){
					$countries[strtolower(Main::clean($country))]=$this->validate(Main::clean($array['target'][$i],3,FALSE));
			  }
			}
			$countries=json_encode($countries);
		}else{
			$countries='';
		}			
		// If URL is already shortened, retrieve it
		if($this->logged_id){
			if($data=$this->db->get("url",array("url"=>"?","userid"=>"?","location"=>"?","pass"=>"?"),array("limit"=>1),array($url,$this->logged_id,$countries,""))){
				return $this->build($data["alias"].$data["custom"]);
				//return array('error' => 0, 'short'=>"{$this->config["url"]}/".$data['alias'].$data["custom"]);
			}
		}
		if(!$this->logged_id && empty($array["password"]) && empty($array["custom"])){
			if($data=$this->db->get("url",array("url"=>"?","userid"=>"?","location"=>"?","pass"=>"?"),array("limit"=>1),array($url,0,"",""))){
					$this->check_history($data["alias"],$data["custom"]);				
					return $this->build($data["alias"].$data["custom"]);
				//return array('error' => 0, 'short'=>"{$this->config["url"]}/".$data['alias'].$data["custom"]);				
			}			
		}				
		// Check custom alias
		if(isset($array["custom"]) && !empty($array["custom"])){
			if(strlen($array["custom"])<3){
				return array('error' => 1, 'msg' => e('Custom alias must be at least 3 characters.'));
			}elseif($this->blacklist_keywords($array["custom"])){
				return array('error' => 1, 'msg' => e('Inappropriate aliases are not allowed.'));
			}elseif($this->db->get("url",array("custom"=>"?"),"",array(Main::slug(Main::clean($array["custom"]))))){
				return array('error' => 1, 'msg' => e('That alias is taken. Please choose another one.'));
			}elseif($this->reserved_alias($array["custom"])){
				return array('error' => 1, 'msg' => e('That alias is reserved. Please choose another one.'));
			}
		}
		
		// Generate and validate alias
		$custom="";
		$alias="";
		$anonid="";
		if(empty($array["custom"])){
			$unique=FALSE;
			$max_loop=100;
			$i=0;
			while (!$unique) {
				if($i>=$max_loop) $this->alias_length=6;
				$alias=Main::strrand($this->alias_length);
				if(!$this->db->get("url",array("alias"=>$alias))) $unique=TRUE;
				$i++;
			}	
		}else{
			$custom=Main::slug(Main::clean($array["custom"]));
		}
		if(!$this->logged()){
			$this->check_history($alias,$custom);
		}
	
		$info=Main::get_meta_data($url);
		$meta_title="";
		$meta_description="";
		if(!empty($info)){
			$meta_title=Main::clean($info['title'],3);
			$meta_description=Main::truncate(Main::clean($info['description'],3));
		}
		$key=array(
			"alias"=>"?",
			"custom"=>"?",
			"url"=>"?",
			"description"=>"?",
			"location"=>"?",
			"date"=>"NOW()",
			"pass"=>"?",
			"meta_title"=>"?",
			"meta_description"=>"?",
			"userid"=>"?",
			"public"=>"?"
			);	
		$values=array($alias,$custom,$url,Main::clean(!empty($array["description"])?$array["description"]:''),$countries,(empty($array["password"])?'':Main::encode($array["password"])),$meta_title,$meta_description,$this->logged_id,"0");
		if($this->db->insert("url",$key,$values)){
			return $this->build($alias.$custom);
			//return array('error' => 0, 'short'=>"{$this->config["url"]}/".$alias.$custom);
		}
		return;
	}
	/**
	 * Check history
	 * @since v2.0
	 */
	protected function check_history($alias,$custom){
		if(Main::cookie("aid")){
			$anonid=Main::cookie("aid");			
			if(!preg_match("~$alias~", $anonid) || !preg_match("~$custom~", $anonid)){
				$ids=explode("-",$anonid);
				if(count($ids)>10){
					$c="";
					for ($i=0; $i < 10; $i++) { 
						$c.="-".$ids[$i];
					}
					Main::cookie("aid","-$alias$custom$c",60*24*365);					
				}else{
					Main::cookie("aid","-$alias$custom$anonid",60*24*365);
				}
			}
		}else{
			Main::cookie("aid","-$alias$custom",60*24*365);      	
		}				
	}
	/**
	 * Bulk URL Shortening
	 * @since v2.0
	 */	
	public function bulk_add(){
		$urls="";
		unset($_POST["url"]);
 	  unset($_POST["custom"]);
 	  unset($_POST["pass"]);
 	  unset($_POST["description"]);
 	  unset($_POST["country"]);
 	  unset($_POST["target"]);

		// Check Captcha
		if(!$this->logged_id){
			$captcha=Main::check_captcha($_POST);
			if($captcha!='ok'){
				return array('error' => 1, 'msg' => $captcha,'html'=>'captcha');
			}
		}	
 	  $url=explode("\n",$_POST["urls"]);
 	  foreach($url as $link){
 	  	if(!empty($link)){
				$short=$this->add(array("url"=>trim($link)));
	    	if(!$short["error"]){
	    		$urls.=$link." => ".$short["short"]." \n";
	    	}else{		    		
	    		$urls.=$link." => ".$short["msg"]." \n";
	    	}
 	  	}
 	  }
 		return array("error"=>0,"confirm"=>1, "short"=> str_replace("\r", "", $urls));	 	  
	}	
	/**
	 * Update Stats
	 * @since v2.0
	 */
	protected function update($delay=15){
		if(Main::bot()) return FALSE;
		$id=Main::clean($_GET["action"]);
		if(isset($_COOKIE["short_$id"])) return FALSE;			

		if($this->db->update("url",array("click"=>"click+1"),"alias=:a OR custom=:a",array(":a"=>$id))){
			setcookie("short_$id",'1',time()+($delay*60),'/');
		}
		if($this->config["tracking"]=="1"){
			// System Analytics
			if(isset($_SERVER["HTTP_REFERER"]) && !empty($_SERVER["HTTP_REFERER"]) && !is_null($_SERVER["HTTP_REFERER"])){
				$referer=$_SERVER["HTTP_REFERER"];
			}else{
				$referer="direct";
			}
			$array=array($id,Main::clean($_SERVER['REMOTE_ADDR']),$this->country($_SERVER['REMOTE_ADDR']),$referer);
			$this->db->insert("stats",array("short"=>"?","date"=>"NOW()","ip"=>"?","country"=>"?","referer"=>"?"),$array);

		}
		return FALSE;		
	}
	/**
	 * Redirect URL
	 * @todo FINSH
	 * @since v3.0
	 */		
	public function redirect($redirect=TRUE,$pass=FALSE){
		// QR and Screenshot
		if(isset($_GET["do"]) && $_GET["do"]=="qr"){				
			return $this->qr();	
		}
		if(isset($_GET["do"]) && $_GET["do"]=="i"){			
			return $this->image();		
		}			

		// Redirect to stats page
		if(isset($_GET["do"]) && $_GET["do"]=="stat"){
			Main::redirect("{$_GET["action"]}+","","301");
		}
		if($_GET["action"]=="stats") {			
			return $this->stats();
		}

		// Fetch URL info
		if($url=$this->db->get("url","BINARY alias=:id OR BINARY custom=:id",array("limit"=>1),array(":id"=>$_GET["action"]))){ 
			
			// Check if password protected
			if(!$pass && !empty($url["pass"]) && !$this->admin){	
				if($this->logged && $url["userid"]==$this->logged_id) return;
				$this->redirect_pass($url["pass"]);
				return;
			}

			// URL is OK Update stats
			if(!$redirect) return FALSE;
			if(!$url["ads"]) $this->show_ads=FALSE;
			$this->update();	

			// Check if url has meta title and meta description, if not attempt to fetch it
			if(empty($url["meta_title"]) && empty($url["meta_description"])){
				$info=Main::get_meta_data($url["url"]);
				if(!empty($info)){
					$meta_title=Main::clean($info['title'],3);
					$meta_description=Main::truncate(Main::clean($info['description'],3));
					$this->db->update("url",array("meta_title"=>"?","meta_description"=>"?"),array("id"=>$url["id"]),array($meta_title,$meta_description));
				}
			}			
		
			// Check if URL is geotargeted
			if(!empty($url["location"])){
				$location=json_decode($url["location"],TRUE);				
				$country='';
				$get_country=$this->country($_SERVER['REMOTE_ADDR']);
				if($get_country) $country=$get_country;					
				if (isset($location[$country])) {
					$url["url"]=$location[$country];
				}
			}

			// Set Meta info
			if(!empty($url["meta_title"])) Main::set("title",$url["meta_title"]);
			if(!empty($url["meta_description"])) Main::set("description",$url["meta_description"]);
			Main::set("url","{$this->config["url"]}/{$url["alias"]}{$url["custom"]}");
			Main::set("image","{$this->config["url"]}/{$url["alias"]}{$url["custom"]}/i");	
			Main::video($url["url"]);

			if($this->config["frame"]=="1" OR ($this->redirect_override==TRUE && isset($_GET["r"]) && $_GET["r"]=="frame")){
				// Frame
				return $this->frame_page($url);
			}elseif($this->config["frame"]=="2" OR ($this->redirect_override==TRUE && isset($_GET["r"]) && $_GET["r"]=="splash")){
				// Splash
				return $this->splash_page($url);
			}else{
				// Direct
				return $this->direct($url["url"]);
			}
		}
		return $this->_404();		
	}	
	/**
	 * Splash Page
	 * @since v3.0
	 */
	public function splash_page($url=array()){
		if($user_media=$this->db->get("user",array("id"=>$url["userid"]),array("limit"=>1))){
			if(!empty($user_media["splash"]) && $user_media["splash_opt"]) return $this->custom_splash($url["url"],$user_media["splash"],$url["userid"]);
			$url["media"]=$user_media["media"];
		}else{
			$url["media"]=$this->config["show_media"];
		}
		// If is media, show media page
		if(($host=$this->is_media($url["url"])) && $url["media"]){
			if($media=$this->oembed($url["url"],$host)) return $this->media_page($media,$url);		
		}
		// Inject GA code
		if($this->config["tracking"]=="2" && !empty($this->config["analytic"])){					
			Main::add("<script>(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)})(window,document,'script','//www.google-analytics.com/analytics.js','ga');ga('create', '{$this->config["analytic"]}', '{$this->domain($this->config["url"])}');ga('send', 'pageview');</script>","custom",FALSE);
		}			
		// Add timer animation	
		if(!empty($this->config["timer"])){
			//Main::add("<script>setInterval(function(){ window.location='{$url["url"]}';}, ".($this->config["timer"]*1000).");</script>","custom",FALSE);
			Main::add('<script>var count = '.$this->config['timer'].';var countdown = setInterval(function(){$("a.redirect").html(count + " seconds");if (count < 1) {clearInterval(countdown);window.location="'.$url['url'].'";}count--;}, 1000);</script>',"custom",FALSE);					
		}
		include(TEMPLATE."/header.php");
		// If url matches keywords - show an alternate splash page - else show regular
		if(file_exists(TEMPLATE."/splash_alt.php") && $this->blacklist_keywords($url["url"])){
			include(TEMPLATE."/splash_alt.php");
		}else{
			include(TEMPLATE."/splash.php");				
		}
		include(TEMPLATE."/footer.php");
	}
	/**
	 * Media Page
	 * @since v3.0
	 */
	public function media_page($media=array(),$url=array()){		
		if(!isset($media["title"])) $media["title"]=$url["meta_title"];
		if(!isset($media["description"])) $media["description"]=$url["meta_description"];
		$media["click"]=$url["click"];
		$media["shorturl"]=$this->config["url"]."/".$url["alias"].$url["custom"];
		$media["url"]=$url["url"];
  	Main::add($this->config["url"]."/static/zclip.js","script",FALSE);		
		include(TEMPLATE."/header.php");
		include(TEMPLATE."/media.php");
		include(TEMPLATE."/footer.php");
	}	
	/**
	 * Custom Splash Page
	 * @since v3.0
	 */	
	public function custom_splash($url,$splash,$id,$timer=5){
		$data=json_decode($splash,TRUE);
		$data["banner"]=$this->config["url"]."/static/user_content/{$id}_files/{$data["banner"]}";
		$data["avatar"]=$this->config["url"]."/static/user_content/{$id}_files/{$data["avatar"]}";	
		Main::add('<script>var count = "5";var countdown = setInterval(function(){$(".c-countdown span").html(count);if (count < 1) {clearInterval(countdown);window.location="'.$url.'";}count--;}, 1000);</script>',"custom",FALSE);							
		include(TEMPLATE."/header.php");
		include(TEMPLATE."/custom_splash.php");
		include(TEMPLATE."/footer.php");
	}	
	/**
	 * Frame Page
	 * @since v3.0
	 */
	public function frame_page($url=array()){
		// URL cannot be framed, redirect
		if(!$this->url_framed($url["url"])){
			header("Location: {$url["url"]}");
		}
		// If google analtytics, use code
		if($this->config["tracking"]=="2" && !empty($this->config["analytic"])){					
			Main::add("<script>(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)})(window,document,'script','//www.google-analytics.com/analytics.js','ga');ga('create', '{$this->config["analytic"]}', '{$this->domain($this->config["url"])}');ga('send', 'pageview');</script>","custom",FALSE);
		}
		include(TEMPLATE."/frame.php");
	}	
	/**
	 * Direct
	 * @since v3.0
	 */
	public function direct($url){
		// If google analtytics, use code
		if($this->config["tracking"]=="2" && !empty($this->config["analytic"])){					
			echo "<html><head><script>(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)})(window,document,'script','//www.google-analytics.com/analytics.js','ga');ga('create', '{$this->config["analytic"]}', '{$this->domain($this->config["url"])}');ga('send', 'pageview');ga(function() {setTimeout(function() {window.location = '{$url}';}, 250);});</script></head><body></body></html>";
		}else{
			header("Location: {$url}");
		}		
	}	
/**
	 * Is media URL
	 * @since v3.0
	 */
	protected function is_media($url){
		$list=array("youtube","vimeo","dailymotion","funnyordie","collegehumor","vine");    
    preg_match('((http://|https://|www.)([\w-\d]+\.)+[\w-\d]+)',$url, $r);
    $host = @str_replace(".","", $r[2]);
    $pattern=array(
				"youtube" => "youtube\.com\/watch\?v=([^\&\?\/]+)",
				"vimeo" => "vimeo\.com\/([^\&\?\/]+)",
				"vine" => "vine\.co\/v/([^\&\?\/]+)",
				"dailymotion" => "dailymotion\.com\/video/([^\&\?\/]+)",
				"funnyordie" => "funnyordie\.com\/videos/([^\&\?\/]+)",
				"collegehumor" => "collegehumor\.com\/video/([^\&\?\/]+)"
    	);
    if(!in_array($host, $list)) return FALSE;    
    if(!preg_match("~{$pattern[$host]}~", $url)) return FALSE;
   	return $host;    
	}
	/**
	 * Embed Media Information
	 * @since v3.0
	 */
	public function oembed($url,$host){
		$oembed=array(
				"youtube" => "http://www.youtube.com/oembed?url=@",
				"vimeo" => "http://vimeo.com/api/oembed.json?url=@",
				"dailymotion" => "http://www.dailymotion.com/services/oembed?format=json&url=@",
				"funnyordie" => "http://www.funnyordie.com/oembed.json?url=@",
				"collegehumor" => "http://www.collegehumor.com/oembed.json?url=@"
			);
		$custom=array("vine","break","metacafe");
		if(isset($oembed[$host])) {
			if($data=file_get_contents(str_replace("@", $url, $oembed[$host]))){
				$data=json_decode($data,TRUE);
				if(!isset($data["html"]) || empty($data["html"])) return FALSE;
				return $data;				
			}
		}elseif(in_array($host, $custom)){
			return $this->custom_oembed($url,$host);
		}
		return FALSE;
	}
	/**
	 * Custom oEmbed Media Information
	 * @since v3.0
	 */
	public function custom_oembed($url,$host){
		$data=array();
		switch ($host) {
			case 'vine':
				$data=array(
					"provider_name"=>"Vine",
					"html"=>'<iframe class="vine-embed" src="'.$url.'/embed/postcard" width="600" height="600" frameborder="0"></iframe><script async src="//platform.vine.co/static/scripts/embed.js" charset="utf-8"></script>'
				);
			return $data;
			break;
		}
		return FALSE;
	}		
	/**
	 * Validate Password
	 * @since v2.0
	 */		
	public function redirect_pass($password){
		if(isset($_POST["event"]) && $_POST["event"]=="unlock" && Main::validate_csrf_token($_POST["token"])){
			if(!empty($_POST["password"]) && Main::encode($_POST["password"])==$password){
				$this->redirect(TRUE,TRUE);
				return;
			}else{
				Main::redirect(Main::clean($_GET["action"]),array("error",e("Please enter a valid password.")));
				return;
			}
		}
		
		include(TEMPLATE."/header.php");
		include(TEMPLATE."/password.php");
		include(TEMPLATE."/footer.php");		
		return;
	}		
	/**
	 * Stats Page
	 * @since v2.0
	 */		
	public function stats(){
		if($url=$this->db->get("url","BINARY alias=:a OR BINARY custom=:a",array("limit"=>1),array(":a"=>$_GET["id"]))){			
			if($this->admin==FALSE){
				if(!$this->logged){
					if($url["userid"]=="0" && !empty($url["pass"])){
						Main::redirect("",array("error",e("The stats of this URL cannot be accessed.")));
						return;
					}
					if($url["userid"]!="0"){
						Main::redirect("user/login",array("error",e("This URL is private and only the creator can access the stats. If you are the creator, please login to access it.")));
						return;
					}					
				} 
				if($this->logged){
					if($url["userid"]!="0" && $this->logged_id!=$url["userid"]){
						Main::redirect("user",array("error",e("This URL is private and only the creator can access the stats. If you are the creator, please login to access it.")));
						return;						
					}
				}
			}			
			$this->db->get("stats","short=?",array("group"=>"ip"),array($_GET["id"]));
			if($this->db->rowCountAll==0){
				$url["unique_click"]="n/a";
			}else{
				$url["unique_click"]=$this->db->rowCountAll;
			}
			Main::set("title","{$url["meta_title"]} Stats");
			Main::set("description","{$url["meta_description"]}");
			Main::set("url","{$this->config["url"]}/{$url["alias"]}{$url["custom"]}");
			Main::add("{$this->config["url"]}/static/zclip.js","script",FALSE);
			Main::add("http://www.google.com/jsapi","script");
			Main::add("{$this->config["url"]}/analytic?page=".Main::clean($_GET["id"]),"script");
			Main::add("<script type='text/javascript'>$('#ajax-stats').dropkick({change:function(a){regenerate_graph(a)}});</script>","custom",TRUE);			
			include(TEMPLATE."/header.php");
			include(TEMPLATE."/stats.php");
			include(TEMPLATE."/footer.php");	
		}else{
			$this->_404();			
			return;
		}	
	}
	/**
	 * Generate QR
	 * @since v2.1.1
	 */		
	protected function qr(){
		$api_url="http://chart.apis.google.com/chart?chs=149x149&chld=L|0&choe=UTF-8&cht=qr&chl=".urlencode($this->config["url"]."/".$_GET["action"]);
		if(!ini_get('allow_url_fopen')){
			header("Location: $api_url");
			exit;
		}		
		$image=file_get_contents($api_url);
		header('Content-type: image/jpeg;');
		echo $image;
	}
	/**
	 * Generate Thumbnail
	 * @since v3.0
	 */		
	protected function image(){
		$t_url=$this->config["url"]."/404";
		if($url=$this->db->get("url","alias=:a OR custom=:a",array("limit"=>1),array(":a"=>$_GET["action"]))){
			if(empty($url["pass"])){
				$t_url=$url["url"];
			}
		}else{
			return;
		}
		$t_url=urlencode($t_url);
		if($this->thumb_provider=="mshots"){
			$api_url="http://s.wordpress.com/mshots/v1/$t_url?w=400";
		}elseif($this->thumb_provider=="pagepeeker"){
			$api_url="http://api.pagepeeker.com/v2/thumbs.php?size=l&url=$t_url";
		}else{
			$api_url=str_replace("@URL@",$t_url,$this->thumb_provider);
		}		
		if(!ini_get('allow_url_fopen')){
			header("Location: $api_url");
			exit;
		}		
		$image=file_get_contents($api_url);
		header('Content-type: image/jpeg;');
		echo $image;		
	}	
	/**
	 * Build return URL
	 * @since v2.1
	 */
	protected function build($data){
		if(!empty($this->user_domain) && is_null($this->return_url) && $this->validate_domain_names($this->user_domain)) $this->return_url=$this->user_domain;		
		if(!is_null($this->return_url)){
			if(preg_match("~{?}~", $this->return_url)){
				return array("error"=>0,"short"=>str_replace("{?}",$data,$this->return_url));
			}else{
				return array("error"=>0,"short"=>"{$this->return_url}/$data");
			}
		}
		return array("error"=>0,"short"=>"{$this->config["url"]}/$data");
	}
	/**
	 * Validate URL
	 * @since v2.0
	 */	
	protected function validate($url){
			if(empty($url)) return FALSE;		
			if(preg_match('(((http://|https://|ftp://|www.)*?)([\w-\d]+\.)+[\w-\d]+)', $url)) {
				if(!preg_match('(http://|https://|ftp://)',$url)){
					$url="http://$url";
				}
			}else{
				return FALSE;
			}
			if(!filter_var($url, FILTER_VALIDATE_URL)) return FALSE;				
			$url=rtrim($url, '/');
		return $url;
	}
	/**
	 * Check if domain is blacklisted
	 * @since 2.0
	 */
	protected function blacklist_domain($url){
		if(empty($this->config["domain_blacklist"])) return FALSE;
		$array=explode(",",$this->config["domain_blacklist"]);
		foreach ($array as $domain) {
		  if (preg_match("~$domain~i", $url)) {
		  	return TRUE;
		  }
		}
		return FALSE;		
	}
	/**
	 * Check if URL contains blacklisted keywords
	 * @since 2.0
	 */
	protected function blacklist_keywords($url){
		if(!$this->config["keyword_blacklist"]) return FALSE;
		if(empty($this->config["keyword_blacklist"])) {
			$array = array('porn','sex','porno','redtube','4tube','spankwire',
					'xshare','ziporn','naked','pornstar','pussy','fuck','suck','porntube',
					'scriptmaster','warez','scriptmafia','nulled','jigshare','gaaks','newone',
					'intercambiosgratis','scriptease','xtragfx','vivaprogram','kickassgfx',
					'gfxdl','fulltemplatedownload','dlscript','nigger','dick','faggot','cunt','gay',
					'asshole','penis','vagina','motherfucker','fucker','shit','fucked','boobs');
		}else{
			$array=explode(",",$this->config["keyword_blacklist"]);
		}
		foreach ($array as $domain) {
		  if (preg_match("~$domain~i", $url)) {
		  	return TRUE;
		  }
		}
		return FALSE;		
	}	
	/**
	 * Check if domain is safe
	 * @since 2.0
	 */	
	protected function safe($url){
		if(!empty($this->config["safe_browsing"])){
			$api="https://sb-ssl.google.com/safebrowsing/api/lookup?client=api&apikey={$this->config["safe_browsing"]}&appver=1.0&pver=3.0&url=";
			$url=urlencode($url);
			$get=@file_get_contents($api.$url);
			if($get=="malware" || $get=="phishing"){
				return FALSE;
			}

		}	
		return TRUE;				
	}	
	/**
	 * Get country from IP now with GeoIP - must be downloaded separately.
	 * @since v3.0
	 */	
	public function country($ip=NULL,$api=''){
		if(is_null($ip)) $ip=$_SERVER['REMOTE_ADDR'];
		if(empty($this->config["apikey"])){
			if(file_exists(ROOT."/includes/lib/GeoIP.dat") && file_exists(ROOT."/includes/lib/geoip.inc")){
				include(ROOT."/includes/lib/geoip.inc");
				$gi = geoip_open(ROOT."/includes/lib/GeoIP.dat",GEOIP_STANDARD);
				$country=geoip_country_name_by_addr($gi,$ip);
				geoip_close($gi);	
				return $country;
			}
		}
		$content=@file_get_contents("http://api.ipinfodb.com/v3/ip-city/?key={$this->config["apikey"]}&ip=$ip");
		if($content){
			$content=explode(';',$content);
			return strtolower($content["4"]);				
		}
		return FALSE;
	}	
	/**
	 * Check if url can framed
	 * @since v2.0
	 */	
	protected function url_framed($url){
		$array=array("facebook.com","youtube.com","google.com","gmail.com","yahoo.com");
		$domain=$this->domain($url);
		foreach ($array as $u) {
		  if (preg_match("~$u~i", $domain)) {
		  	return FALSE;
		  }
		}
		return TRUE;			
	}
	/**
	 * List of reserved custom alias
	 */
	protected function reserved_alias($alias){
		$array=array("page","api","user","index","analytics","contact","admin","profile","bundle");
		if(in_array($alias,$array)) return TRUE;
		return FALSE;
	}
	/**
	 * Generate API
	 * @since v2.2
	 */
	public function api(){
		if(!$this->config['api']) {
			echo $this->return_api(array("error"=>"1", "msg"=>"The API service has been disabled."));
			return;
		}
		// Get Long URL + stats
		if(isset($_GET["short"]) & !empty($_GET["short"])){		
			preg_match("~(.*)://(.*)/(.*)~", $_GET["short"],$short);
			if(empty($short["3"]) || !$url=$this->db->get("url","alias=:a OR custom=:a",array("limit"=>1),array(":a"=>$short["3"]))) {
				echo $this->return_api(array("error"=>"1", "msg"=>"Invalid short URL."));
			}
			// Show Long URL for anon
			if($url["userid"] && !isset($_GET["api"])) {
				echo $this->return_api(array("error"=>"1", "msg"=>"This is a private URL. A valid API key is required to view details."));
				return;
			}else{
				echo $this->return_api(array("error"=>"0","long"=>$url["url"],"clicks"=>$url["click"],"date"=>$url["date"]),$url["url"]);
						return;				
			}
			// Show Short URL info for private URL
			if(isset($_GET["api"]) && !empty($_GET["api"])){
				if($user=$this->db->get("user",array("api"=>"?"),array("limit"=>1),array($_GET["api"]))){
					if($user["id"]==$url["userid"]){
						echo $this->return_api(array("error"=>"0","long"=>$url["url"],"clicks"=>$url["click"],"date"=>$url["date"]),$url["url"]);
						return;
					}
				}else{
					echo $this->return_api(array("error"=>"1", "msg"=>"This is a private URL. A valid API key is required to view details."));
					return;
				}
			}
			return;
		}
		// Shorten Long URl
		if(empty($_GET["api"]) || !$user=$this->db->get("user",array("api"=>"?"),array("limit"=>1),array($_GET["api"]))){
			echo $this->return_api(array("error"=>"1", "msg"=>"API key cannot be validated."));
			return;
		}else{
			$this->logged_id=$user["id"];
		}
		// Validate URL
		if(!isset($_GET["url"]) || empty($_GET["url"])){
			echo $this->return_api(array("error"=>"1", "msg"=>"This URL is not valid."));
			return;
		}else{
			$array["url"]=Main::clean($_GET["url"]);
		}
		// Set custom Alias
		if(isset($_GET["custom"]) && !empty($_GET["custom"])){
			if($this->db->get("url",array("custom"=>"?"),array("limit"=>1),array(Main::clean($_GET["custom"])))){
				echo $this->return_api(array("error"=>"1","msg"=>"This custom alias is already taken."));
				return ;
			}else{
				$array["custom"]=Main::clean($_GET["custom"]);
			}
		}
		// Everything is ok, let's add it
		$res=$this->add($array);
		echo $this->return_api($res,$res["short"]);
	}
	protected function return_api($array,$text=""){
		if(isset($_GET["callback"])){
			header("Content-type: application/javascript");
			return "{$_GET["callback"]}(".json_encode($array).")";
		}
		if(isset($_GET["format"]) && $_GET["format"]=="text"){
			return $text;
		}
		header("Content-type: application/javascript");		
		return json_encode($array);
	}	
}
?>